import DataLayer.DataMapper;
import DataLayer.testClasses;
import FunctionLayer.Datahandler;
import FunctionLayer.PipelineJMWESingleton;
import FunctionLayer.StanfordParser.SentimentAnalyzerTestDynamicTesting;
import edu.mit.jmwe.data.IMWE;
import edu.mit.jmwe.data.IToken;
import edu.stanford.nlp.ie.AbstractSequenceClassifier;
import edu.stanford.nlp.ie.crf.CRFClassifier;
import edu.stanford.nlp.ling.CoreAnnotations;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.ling.TaggedWord;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.CoreDocument;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.tagger.maxent.MaxentTagger;
import edu.stanford.nlp.trees.*;
import edu.stanford.nlp.util.CoreMap;
import org.ejml.simple.SimpleMatrix;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.junit.Assert;
//import org.junit.Test;
import org.junit.jupiter.api.Test;

import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.*;

public class TestJunit {

    private String taggerPath = "edu/stanford/nlp/models/pos-tagger/english-left3words-distsim.tagger";
    private MaxentTagger tagger = new MaxentTagger(taggerPath);
    private GrammaticalStructureFactory gsf = initiateGrammaticalStructureFactory();

    private JSONParser parser = new JSONParser();
    private JSONObject dataFromJson = null;

    String nerModel = "edu/stanford/nlp/models/ner/english.all.3class.distsim.crf.ser.gz";
    AbstractSequenceClassifier<CoreLabel> classifier = CRFClassifier.
            getClassifierNoExceptions(nerModel);

    public GrammaticalStructureFactory initiateGrammaticalStructureFactory() {
        String lexParserEnglishPCFG = "edu/stanford/nlp/models/lexparser/englishPCFG.ser.gz";
        LexicalizedParser lp = LexicalizedParser.
                loadModel(lexParserEnglishPCFG, "-maxLength", "100");
        TreebankLanguagePack langpack = lp.getOp().langpack();
        return langpack.grammaticalStructureFactory();
    }

    public Double testCall(testClasses testClass, StanfordCoreNLP stanfordCoreNLP, StanfordCoreNLP stanfordCoreNLPSentiment, JSONObject dataRandom,
                           String tableTestType, boolean testAll, int indexCounter) {
        String sent1 = testClass.getSentence1();
        String sent2 = testClass.getSentence2();

        //System.out.println("sent1: " + sent1);
        //System.out.println("sent2: " + sent2);

        ArrayList<String> concurrentRelations = new ArrayList<String>();
        Annotation jmweAnnotationF = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(sent1);
        Annotation jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(sent2);

        Integer tokenizeCountingF = null;
        List<List<TaggedWord>> taggedWordListF = null;
        List<List<TaggedWord>> taggedWordList1 = null;
        ArrayList<String> retrieveTGWListF = null;
        java.util.ArrayList<String> retrieveTGWList1 = null;
        List<CoreMap> sentencesF = null;
        List<CoreMap> sentence1 = null;
        List<CoreMap> sentencesSentimentF = null;
        List<CoreMap> sentenceSentiment1 = null;
        List<CoreMap> coreMaps1 = jmweAnnotationF.get(CoreAnnotations.SentencesAnnotation.class);
        ArrayList<Tree> treesF = null;
        ArrayList<Tree> trees1 = null;
        ArrayList<GrammaticalStructure> grammaticalStructuresF = null;
        ArrayList<GrammaticalStructure> grammaticalStructures1 = null;
        ArrayList<TypedDependency> typedDependenciesF = null;
        ArrayList<Integer> rnnCoreAnnotationsPredictedF = null;
        ArrayList<SimpleMatrix> simpleMatricesF = null;
        ArrayList<SimpleMatrix> simpleMatricesNodevectorsF = null;
        ArrayList<?> listF = null;
        Integer longestF = null;
        Integer sentimentLongestF = null;
        List<IMWE<IToken>> imwesF = null;
        Integer InflectedCounterNegativeF = null;
        Integer InflectedCounterPositiveF = null;
        ArrayList<String> tokenEntryF = null;
        Integer MarkedContinuousCounterF = null;
        Integer UnmarkedPatternCounterF = null;
        ArrayList<String> strTokensIpartFormF = null;
        ArrayList<String> tokenFormsF = null;
        ArrayList<String> strTokenEntryGetPOSF = null;
        ArrayList<Integer> intTokenEntyCountsF = null;
        ArrayList<String> ITokenTagsF = null;
        ArrayList<String> strTokenStemsF = null;
        Integer AnotatorcounterF = null;
        Integer TokensCounterF = null;
        ArrayList<String> entityTokenTagsF = null;
        ArrayList<String> nerEntitiesF = null;
        ArrayList<String> nerEntitiesTypeF = null;
        ArrayList<String> stopWordTokenF = null;
        ArrayList<String> stopWordLemmaF = null;
        Integer PairCounterF = null;

        java.util.ArrayList<TypedDependency> typedDependencies1 = null;
        ArrayList<Integer> rnnCoreAnnotationsPredicted1 = null;
        ArrayList<SimpleMatrix> simpleMatrices1 = null;
        ArrayList<SimpleMatrix> simpleMatricesNodevectors1 = null;
        List<?> list1 = null;
        Integer longest1 = null;
        Integer sentimentLongest1 = null;
        List<IMWE<IToken>> imwes1 = null;
        Integer InflectedCounterNegative1 = null;
        Integer InflectedCounterPositive1 = null;
        ArrayList<String> tokenEntry1 = null;
        Integer MarkedContinuousCounter1 = null;
        Integer UnmarkedPatternCounter1 = null;
        ArrayList<String> strTokensIpartForm1 = null;
        ArrayList<String> tokenForms1 = null;
        ArrayList<String> strTokenEntryGetPOS1 = null;
        ArrayList<Integer> intTokenEntyCounts1 = null;
        ArrayList<String> ITokenTags1 = null;
        ArrayList<String> strTokenStems1 = null;
        Integer Anotatorcounter1 = null;
        Integer TokensCounter1 = null;
        ArrayList<String> entityTokenTags1 = null;
        ArrayList<String> nerEntities1 = null;
        ArrayList<String> nerEntitiesType1 = null;
        ArrayList<String> stopWordToken1 = null;
        ArrayList<String> stopWordLemma1 = null;
        Integer PairCounter1 = null;
        List<CoreMap> coreMaps2 = jmweAnnotation.get(CoreAnnotations.SentencesAnnotation.class);



        Annotation strAnno = new Annotation(sent1);
        strAnno.compact();
        stanfordCoreNLP.annotate(strAnno);

        Annotation strAnno2 = new Annotation(sent2);
        strAnno2.compact();
        stanfordCoreNLP.annotate(strAnno2);



        Annotation strAnnoSentiment = new Annotation(sent1);
        strAnnoSentiment.compact();
        stanfordCoreNLPSentiment.annotate(strAnnoSentiment);

        Annotation strAnnoSentimen2 = new Annotation(sent2);
        strAnnoSentimen2.compact();
        stanfordCoreNLPSentiment.annotate(strAnnoSentimen2);



        Annotation annotation = new Annotation(sent1);
        stanfordCoreNLP.annotate(annotation);
        CoreDocument coreDocument = new CoreDocument(annotation);

        annotation = new Annotation(sent2);
        stanfordCoreNLP.annotate(annotation);
        CoreDocument coreDocument1 = new CoreDocument(annotation);


        Integer tokenizeCounting = null;

        SentimentAnalyzerTestDynamicTesting sentimentAnalyzerTest = new SentimentAnalyzerTestDynamicTesting(sent1, sent2,
                coreMaps1, coreMaps2, strAnno,
                strAnno2, strAnnoSentiment,
                strAnnoSentimen2, coreDocument,
                coreDocument1,
                tagger, gsf, classifier, tokenizeCounting, tokenizeCountingF,
                taggedWordListF, taggedWordList1, retrieveTGWListF, retrieveTGWList1,
                sentencesF, sentence1, sentencesSentimentF, sentenceSentiment1, treesF, trees1,
                grammaticalStructuresF, grammaticalStructures1, typedDependenciesF,
                typedDependencies1, rnnCoreAnnotationsPredictedF, rnnCoreAnnotationsPredicted1,
                simpleMatricesF, simpleMatrices1, simpleMatricesNodevectorsF, simpleMatricesNodevectors1,
                listF, list1, longestF, longest1, sentimentLongestF, sentimentLongest1, imwesF,
                imwes1, InflectedCounterNegativeF, InflectedCounterNegative1, InflectedCounterPositiveF,
                InflectedCounterPositive1, tokenEntryF, tokenEntry1, MarkedContinuousCounterF,
                MarkedContinuousCounter1, UnmarkedPatternCounterF, UnmarkedPatternCounter1,
                strTokensIpartFormF, strTokensIpartForm1, tokenFormsF, tokenForms1,
                strTokenEntryGetPOSF, strTokenEntryGetPOS1, intTokenEntyCountsF,
                intTokenEntyCounts1, ITokenTagsF, ITokenTags1, strTokenStemsF, strTokenStems1,
                AnotatorcounterF, Anotatorcounter1, TokensCounterF, TokensCounter1,
                entityTokenTagsF, entityTokenTags1, nerEntitiesF, nerEntities1, nerEntitiesTypeF,
                nerEntitiesType1, stopWordTokenF, stopWordToken1, stopWordLemmaF, stopWordLemma1,
                PairCounterF, PairCounter1, dataRandom, !testAll); //testAll = True -> testingfunction = false

        if (testAll) {
            Double score = sentimentAnalyzerTest.callSMX();
            return score;
        }

        //Just do all the fucking rest, put them in small individual tables, get each function to pass. use that result. see if it can pass combined afterwards.
        Double score = 0.0;
        //there should be no null values cause calling validateStringCaches().
        if (tableTestType.equals("tokensCounterScoring")) {
            score = sentimentAnalyzerTest.tokensCounterScoring(0.0, sentimentAnalyzerTest.getTokensCounterF(), sentimentAnalyzerTest.getTokensCounter1());
        }
        else if (tableTestType.equals("stopWordTokenLemmaScoring"))
        {
            score = sentimentAnalyzerTest.stopWordTokenLemmaScoring(0.0, sentimentAnalyzerTest.getStopWordTokenF(), sentimentAnalyzerTest.getStopWordToken1(),
                    sentimentAnalyzerTest.getStopWordLemmaF(), sentimentAnalyzerTest.getStopWordLemma1());
        }
        else if (tableTestType.equals("simpleRNNMaxtrixVectors"))
        {
            //System.out.println("sentimentAnalyzerTest.getSimpleMatricesNodevectorsF(): " + sentimentAnalyzerTest.getSimpleMatricesNodevectorsF());
            //System.out.println("sentimentAnalyzerTest.getSimpleMatricesNodevectors1(): " + sentimentAnalyzerTest.getSimpleMatricesNodevectors1());
            score = sentimentAnalyzerTest.simpleRNNMaxtrixVectors(score, sentimentAnalyzerTest.getSimpleMatricesNodevectorsF(), sentimentAnalyzerTest.getSimpleMatricesNodevectors1());
        }

        return score;
    }

    //@RetryingTest(500)
    @Test
    public void testScoring() {
        Set<JSONObject> randomDataSet = null;

        //is only used when testing specific function, not used when testing all.
        //String testCaseFunction = "tokensCounterScoring";
        //String testCaseFunction = "stopWordTokenLemmaScoring";
        String testCaseFunction = "simpleRNNMaxtrixVectors";
        boolean testAll = false; //put to false when testing a specific function instead.
        if (!testAll) {
            Set<JSONObject> jsonObjects = DataMapper.pickHighestProgression(testCaseFunction);
            if (jsonObjects.isEmpty()) {
                //we start from the bottom with random data.
                LoadDataFromJson();
                randomDataSet = ModifyDataRandomly(testCaseFunction);
            }
            else {
                //we continue from the last highest iteration.
                randomDataSet = miscData(jsonObjects, testCaseFunction);
            }
            DataMapper.WriteDataToSpecificFunction(randomDataSet, testCaseFunction);
            randomDataSet = DataMapper.SelectRandomDataNotProcessed(testCaseFunction);
        }
        else {
            //select 1 row from all tables where passed_all_test_cases. comment out retryingtest when doing this here.
        }

        Datahandler datahandler = new Datahandler();
        PipelineJMWESingleton.getINSTANCE();
        StanfordCoreNLP stanfordCoreNLP = datahandler.pipeLineSetUp();
        StanfordCoreNLP stanfordCoreNLPSentiment = datahandler.shiftReduceParserInitiate();

        //running all tests
        List<testClasses> testClassesListAll = DataMapper.GetAllTestsCases();
        List<testClasses> testClassesSpecificFunction = DataMapper.GetFunctionTestCases();

        boolean passedtests = false;
        int counter = 0;

        int max_index_counter_tests_passed = DataMapper.get_index_counter_tests_passed(testCaseFunction, 0);

        String comperator_for_score_for_failing_testcase = DataMapper.getMaxIndexComparator(max_index_counter_tests_passed);
        Double bestScore = DataMapper.getbestScoreFormax_index_counter_tests_passed(max_index_counter_tests_passed, testCaseFunction, comperator_for_score_for_failing_testcase);


        for (JSONObject dataRandom : randomDataSet) {
            if (counter % 100 == 0)
            {
                System.out.println("counter: " + counter + "/" + randomDataSet.size());
            }
            if (!testAll)
            {
                //here we run tests for individual functions.
                if (passedAllTests(dataRandom, testClassesSpecificFunction, stanfordCoreNLP, stanfordCoreNLPSentiment, testCaseFunction, testAll,
                        max_index_counter_tests_passed)) {
                    DataMapper.MarkSuccessfullFunctionData(testCaseFunction, (Integer) dataRandom.get("rowid"));
                    System.out.println("passed all.");
                    passedtests = true;
                    break;
                }
            }
            else {
                //just curious if it might pass all original test cases afterwards.
                if (passedAllTests(dataRandom, testClassesListAll, stanfordCoreNLP, stanfordCoreNLPSentiment, testCaseFunction, testAll,
                        max_index_counter_tests_passed)) {
                    passedtests = true;
                }
            }
            counter++;
        }
        if (!testAll) {
            DataMapper.UpdateProcessed(testCaseFunction, randomDataSet, max_index_counter_tests_passed, bestScore, comperator_for_score_for_failing_testcase);
            DataMapper.UpdateModifiedKeyForRowID(randomDataSet, testCaseFunction);
        }
        if (!passedtests){
            Assert.assertTrue(1 > 800.0);
        }
    }

    private boolean passedAllTests(JSONObject dataRandom, List<testClasses> testClassesList, StanfordCoreNLP stanfordCoreNLP, StanfordCoreNLP stanfordCoreNLPSentiment,
                                   String tableTestType, boolean testAll, int max_index_counter_tests_passed) {
        int indexCounter = 0;
        dataRandom.put("index_counter_tests_passed", indexCounter); //indicating how far a run got.
        for (testClasses testClass : testClassesList)
        {
            Double PerformTesting = testCall(testClass, stanfordCoreNLP, stanfordCoreNLPSentiment, dataRandom, tableTestType, testAll, indexCounter);
            if (PerformTesting.isNaN() || PerformTesting == null)
            {
                dataRandom.put("index_counter_tests_passed", 0); // we put it to 0 to indicate that it completely fucked up and is no useable.
                return false;
            }
            /*
            if (indexCounter > 46 && PerformTesting < 120.0)
            {
                System.out.println("indexCounter: " + indexCounter);
                System.out.println("str1: " + testClass.getSentence1());
                System.out.println("str2: " + testClass.getSentence2());
                System.out.println("getScore: " + testClass.getScore());
                System.out.println("getComparator: " + testClass.getComparator());
                System.out.println("PerformTesting: " + PerformTesting);
            }
             */
            //update score when we hit the failing test case or were to surpass it.
            if (indexCounter >= max_index_counter_tests_passed)
            {
                dataRandom.put("failed_testcase_score", PerformTesting);
            }
            testClassesList.get(indexCounter).setResultScore(PerformTesting);
            //perfomrfittingless is only done when running the full test case, not individual functions.
            if (testClass.isPerformTestingFittingLess())
            {
                testClasses testClassPerformFittingBetter = testClassesList.get(indexCounter - 1);
                if (testClass.getComparator().equals("<")) {
                    //passing this condition always requires the current score to give a worse result than the previous one because only one of the sentences is replaced and
                    //is meant to act inferior compared to the previous sentence used instead.
                    if (PerformTesting.intValue() > testClassPerformFittingBetter.getResultScore()) {
                        return false;
                    }
                }
                else if (testClass.getComparator().equals(">")) {
                    //passing this condition always requires the current score to give a better result than the previous
                    if (PerformTesting.intValue() < testClassPerformFittingBetter.getResultScore()) {
                        return false;
                    }
                }
            }
            else if (testClass.getComparator().equals("<"))
            {
                //if getscore 1 and PerformTesting 500 then we fail
                if (PerformTesting.intValue() > testClass.getScore())
                {
                    return false;
                }
            }
            else if (testClass.getComparator().equals(">"))
            {
                //if getscore -1 and PerformTesting -500 then we fail
                if (PerformTesting.intValue() < testClass.getScore())
                {
                    return false;
                }
            }
            //some cases have "no operation" since we just need the result of them for comparing to the next result.
            //only happens on full test run, not when doing functions.
            indexCounter++;
            //System.out.println("indexCounter: " + indexCounter);
            dataRandom.put("index_counter_tests_passed", indexCounter); //indicating how far a run got.
        }
        return true;
    }

    private void LoadDataFromJson() {
        try {
            dataFromJson = (JSONObject) parser.parse(
                    new FileReader("/mnt/hdd/home/christian/content/sourcemod_plugins_and_extensions/addons/sourcemod/scripting/gogs/ArtificialAutism/dynamicScore.json"));//path to the JSON file.
        } catch (IOException e) {
            try {
                String whoami = System.getProperty("user.name");
                dataFromJson = (JSONObject) parser.parse(
                        new FileReader("/home/" + whoami + "/autism_bot_number_crunching/dynamicScore.json"));//path to the JSON file.
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            } catch (ParseException ex) {
                throw new RuntimeException(ex);
            }
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    private Set<JSONObject> ModifyDataRandomly(String testCaseFunction) {
        Set<JSONObject> randomDataSet = new HashSet<>();
        for (int i = 0; i < 1; i++)
        {
            JSONObject object = new JSONObject();
            for (Object key : dataFromJson.keySet())
            {
                if (!key.toString().startsWith(testCaseFunction + "Double_param") &&
                        !key.toString().startsWith(testCaseFunction + "_param")) continue;
                Object intValue = dataFromJson.get(key);
                if (key.toString().startsWith("simpleRNNMaxtrixVectorsDouble") || key.toString().startsWith("simpleRNNMatrixCalculationsDouble")
                        || key.toString().startsWith("typeDependenciesGrammaticalRelationDouble") || key.toString().startsWith("iterateTreesDouble")) {
                    double doubleVal = (double) intValue;
                    Double RandomUpperBound = (doubleVal) * (new Random().nextInt(10) + 0.1);
                    double randomVal = RandomUpperBound * new Random().nextDouble();
                    object.put(key, randomVal + 0.1);
                } else {
                    intValue = ((Long) intValue).intValue(); //well needing this conversion is fucking scuffed
                    int RandomUpperBound = ((int) intValue + 1) * (new Random().nextInt(10) + 1);
                    int RandomLowerBound = (int) intValue / 2;
                    object.put(key, new Random().nextInt((int) (RandomUpperBound - RandomLowerBound)) + RandomLowerBound);
                }
            }
            randomDataSet.add(object);
        }

        Set<JSONObject> randomDataSetToReturn = miscData(randomDataSet, testCaseFunction);
        return randomDataSetToReturn;
    }

    private JSONObject getCloneRandomShuffled(List<Object> keys, JSONObject clone)
    {
        int keyIndex = 0;
        int splitAmount = new Random().nextInt(2, clone.keySet().size() / 2); //determines how many random values may go into each iteration.

        for (Object key : keys) {
            int negativeOrPositive = new Random().nextInt(3);
            if (key.toString().startsWith("simpleRNNMaxtrixVectorsDouble") || key.toString().startsWith("simpleRNNMatrixCalculationsDouble")
                    || key.toString().startsWith("typeDependenciesGrammaticalRelationDouble") || key.toString().startsWith("iterateTreesDouble")) {
                Double doubleValue1 = (Double) clone.get(key);
                DecimalFormat newFormat = new DecimalFormat("#.###");
                newFormat.setDecimalFormatSymbols(DecimalFormatSymbols.getInstance(Locale.ENGLISH));
                if (negativeOrPositive == 0) { //values below minus 1 million would be silly
                    double newValue1 =  doubleValue1 - (doubleValue1 * new Random().nextDouble(0.0, 25));
                    if (newValue1 > -1000000) {
                        clone.put(key, Double.valueOf(newFormat.format(newValue1)));
                    }
                } else if (negativeOrPositive == 1) { //values over 1 million would be silly
                    double newValue1 =  doubleValue1 + (doubleValue1 * new Random().nextDouble(0.0,25));
                    if (newValue1 < 1000000) {
                        clone.put(key, Double.valueOf(newFormat.format(newValue1)));
                    }
                } else {
                    //we just truncate spaces are comma
                    double newValue1 = Double.valueOf(newFormat.format(doubleValue1));
                    clone.put(key, newValue1);
                }
            }
            else {
                int intValue1 = (int) clone.get(key);
                if (negativeOrPositive == 0) { //values below minus 1 million would be silly
                    int i = intValue1 - (int) (intValue1 * new Random().nextDouble(0.0, 25));
                    if (i > -1000000) {
                        clone.put(key, i);
                    }
                } else if (negativeOrPositive == 1) { //values over 1 million would be silly
                    int i = intValue1 + (int) (intValue1 * new Random().nextDouble(0.0, 25));
                    if (i < 1000000) {
                        clone.put(key, i);
                    }
                } else {
                    //we dont change anything about the key.
                    clone.put(key, intValue1);
                }
            }
            keyIndex++;
            if (keyIndex % splitAmount == 0 && keyIndex > 0) {
                break;
            }
        }
        return clone;
    }

    private Set<JSONObject> miscData(Set<JSONObject> randomDataSet, String testCaseFunction){
        Set<JSONObject> randomDataSetToReturn = new HashSet<>();

        int max_index_counter_tests_passed = DataMapper.get_index_counter_tests_passed(testCaseFunction, 0);
        boolean maxIndexCounterTestsPassedCount = DataMapper.getMaxIndex_counter_tests_passedCount(max_index_counter_tests_passed, testCaseFunction);

        HashMap<Integer, ArrayList<Integer>>  arr = DataMapper.get_parameter_generations(randomDataSet, testCaseFunction);

        for (int i = 0; i < 2; i++) {
            for (JSONObject data : randomDataSet) {
                List<Object> keys = new ArrayList<>(data.keySet());
                Collections.shuffle(keys);
                //first approach if less than 1000 rows for the particular index_counter_tests_passed. we just randomly generate some data.
                if (maxIndexCounterTestsPassedCount) {
                    randomDataSetToReturn.add(getCloneRandomShuffled(keys, (JSONObject) data.clone()));
                } else {
                    JSONObject rowid = compareDataTo(keys, (JSONObject) data.clone(), arr.get(data.get("rowid")), testCaseFunction, true);
                    JSONObject rowid_negative = compareDataTo(keys, (JSONObject) data.clone(), arr.get(data.get("rowid")), testCaseFunction, false);
                    if (rowid != null) {
                        randomDataSetToReturn.add(rowid);
                        randomDataSetToReturn.add(rowid_negative);
                    }
                }
            }
        }
        System.out.println("randomDataSetToReturn size: " + randomDataSetToReturn.size());
        return randomDataSetToReturn;
    }

    private JSONObject compareDataTo(List<Object> keys, JSONObject clone, ArrayList<Integer> arr, String testCaseFunction, boolean isPlussed) {
        int keyIndex = 0;
        int splitAmount = 1;

        boolean modifiedKey = false;
        for (Object key : keys) {
            boolean alreadyDone = false;
            for (int i : arr)
            {
                if (key.toString().endsWith("param" + key))
                {
                    alreadyDone = true;
                    break;
                }
            }
            if (alreadyDone)
            {
                continue;
            }
            modifiedKey = true;

            if (key.toString().startsWith("simpleRNNMaxtrixVectorsDouble") || key.toString().startsWith("simpleRNNMatrixCalculationsDouble")
                    || key.toString().startsWith("typeDependenciesGrammaticalRelationDouble") || key.toString().startsWith("iterateTreesDouble")) {
                Double doubleValue1 = (Double) clone.get(key);
                DecimalFormat newFormat = new DecimalFormat("#.###");
                newFormat.setDecimalFormatSymbols(DecimalFormatSymbols.getInstance(Locale.ENGLISH));
                Double v = Double.valueOf(newFormat.format(doubleValue1));
                if (isPlussed) {
                    clone.put(key, v + (v * 0.10));
                }
                else {
                    clone.put(key, v - (v * 0.10));
                }
            }
            else {
                int intValue1 = (int) clone.get(key);
                if (isPlussed) {
                    clone.put(key, (int) (intValue1 + (intValue1 * 0.10)));
                }
                else {
                    clone.put(key, (int) (intValue1 - (intValue1 * 0.10)));
                }
            }
            clone.put("modified_key", key);
            keyIndex++; //maybe remove keyindex and splitamount again.
            if (keyIndex % splitAmount == 0) {
                break;
            }
        }
        if (!modifiedKey)
        {
            DataMapper.deleteRow((int) clone.get("rowid"), testCaseFunction);
            return null;
        }
        return clone;
    }
}

